<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2017 Amasty (https://www.amasty.com)
 * @package Amasty_Flags
 */


namespace Amasty\Flags\Controller\Adminhtml\FlagAssign;

use Amasty\Flags\Controller\Adminhtml\FlagAssign as FlagAssignAction;

class Assign extends FlagAssignAction
{
    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    private $resultJsonFactory;
    /**
     * @var \Amasty\Flags\Model\ResourceModel\Order\Flag
     */
    private $flagResource;

    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Amasty\Flags\Model\ResourceModel\Order\Flag $flagResource
    ) {
        parent::__construct($context);
        $this->resultJsonFactory = $resultJsonFactory;
        $this->flagResource = $flagResource;
    }

    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Json $response */
        $response = $this->resultJsonFactory->create();

        try {
            $orderId = +$this->getRequest()->getParam('orderId');
            $columnId = +$this->getRequest()->getParam('columnId');
            $flag = $this->getRequest()->getParam('flag');

            if (!$orderId || !$columnId) {
                throw new \Exception(__('Bad request'));
            }

            if (is_array($flag)) {
                $note = isset($flag['note']) ? $flag['note'] : null;
                $this->flagResource->assign($orderId, $columnId, $flag['id'], $note);
            } else {
                $this->flagResource->unassign($orderId, $columnId);
            }
        } catch (\Exception $e) {
            $response->setData([
                'error' => true,
                'message' => $e->getMessage()
            ]);
        }

        return $response;
    }
}
