<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2017 Amasty (https://www.amasty.com)
 * @package Amasty_Shopby
 */


namespace Amasty\Shopby\Plugin\Index;

use Magento\CatalogSearch\Model\Indexer\Fulltext\Action\DataProvider;
use Amasty\Shopby\Helper\Group as GroupHelper;

class IndexerFulltextDataProviderPlugin
{
    /**
     * @var GroupHelper
     */
    private $groupHelper;

    /** @var array|null  */
    private $groupedOptions;

    public function __construct(GroupHelper $groupHelper)
    {
        $this->groupHelper = $groupHelper;
    }

    /**
     * @param DataProvider $subject
     * @param array $indexData
     * @return array
     */
    public function afterGetProductAttributes(DataProvider $subject, array $indexData)
    {
        $indexData = $this->addGroupedToIndexData($indexData);

        return $indexData;
    }

    /**
     * @param array $indexData
     * @return array
     */
    private function addGroupedToIndexData(array $indexData)
    {
        $groupedOptions = $this->getGroupedOptions();
        foreach ($groupedOptions as $attributeId => $optionData) {
            $allAttributeOptionsContainedInGroups = array_keys($optionData);
            foreach ($indexData as &$product) {
                if (isset($product[$attributeId])) {
                    $productOptions = explode(',', $product[$attributeId]);
                    $intersectedOptionIds = array_intersect($allAttributeOptionsContainedInGroups, $productOptions);
                    if (!$intersectedOptionIds) {
                        continue;
                    }

                    $intersectedGroupedData = array_intersect_key($optionData, array_flip($intersectedOptionIds));
                    if (count($intersectedGroupedData)) {
                        $gropedValues = array_unique(array_merge(...$intersectedGroupedData));
                    } else {
                        $gropedValues = [];
                    }

                    $product[$attributeId] .= ',' . implode(',', $gropedValues);
                }
            }
        }

        return $indexData;
    }

    /**
     * @return array
     */
    private function getGroupedOptions()
    {
        if ($this->groupedOptions === null) {
            /** @var \Amasty\Shopby\Model\ResourceModel\GroupAttr\Collection $groupedCollection */
            $groupedCollection = $this->groupHelper->getGroupCollection();
            $groupedCollection
                ->addFieldToSelect(['attribute_id', 'group_code'])
                ->joinOptions()
                ->getSelect()
                ->columns('group_concat(`aagao`.`option_id`) as options')
                ->group('group_id');
            $fetched = $groupedCollection->getConnection()->fetchAll($groupedCollection->getSelect());

            $this->groupedOptions = [];
            foreach ($fetched as $group) {
                foreach (explode(',', $group['options']) as $attributeOptionId) {
                    $this->groupedOptions[$group['attribute_id']][$attributeOptionId][] = $group['group_code'];
                }
            }
        }

        return $this->groupedOptions;
    }
}
